/* mbed Si4735 Library
 * Brett Wilson and Brett Berry
 * Georgia Tech ECE 4180
 * Note: this code has only been tested for FM. 
 * Ported from ...

 * Arduino Si4735 Library
 * Written by Ryan Owens for SparkFun Electronics
 * 5/17/11
 *
 * This library is for use with the SparkFun Si4735 Shield
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 *
 * See the header file for better function documentation.
 *
 * See the example sketches to learn how to use the library in your code.
*/

#include "Si4735.h"
#include "mbed.h"

//This is just a constructor.
Si4735::Si4735(PinName sda, PinName scl, PinName RST_) :
    _RST_(RST_) {
    i2c_ = new I2C(sda, scl);
    i2c_->frequency(100000);
}


void Si4735::begin(char mode){
    _mode = mode;
    //Set the initial volume level.
    _currentVolume=63;

    // Reset the Si4735
    _RST_ = 0;
    wait(.8);
    _RST_ = 1;
    wait(.2);   
    
    //Send the POWER_UP command
    switch(_mode){
        case FM:
            sprintf(command, "%c%c%c", POWER_UP, 0x10, 0x05);
            break;
        case AM:
        case SW:
        case LW:
            sprintf(command, "%c%c%c", POWER_UP, 0x11, 0x05);
            break;
        default:
            return;
    }
    sendCommand(command, 3);
    wait(.2);
    
    sprintf(command, "%c%c%c", FM_AGC_OVERRIDE, 0x01, 0x00);    // AGC disabled
    sendCommand(command, 3);
    wait(.1);
   
    getAGCstatus();
    wait(.1);
    
    // Set the RCLK to 32768Hz
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x02, 0x01, 0x80, 0x00);
    sendCommand(command, 6);
    wait(.2);

    //Split the desired frequency into two character for use in the
    //set frequency command.
    char highByte = Frequency >> 8;
    char lowByte = Frequency & 0x00FF;
        
    // Set default frequency to 87.6 MHz FM
//    sprintf(command, "%c%c%c%c", FM_TUNE_FREQ, 0x00, 0x22, 0x38);
    sprintf(command, "%c%c%c%c", FM_TUNE_FREQ, 0x00, highByte, lowByte);
    sendCommand(command, 4);
    wait(.2);
    
    //Set the volume to the current value.
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x40, 0x00, 0x00, _currentVolume);
    sendCommand(command, 6);
    wait(.2);

    //Disable Mute
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x40, 0x01, 0x00, 0x00);
    sendCommand(command, 6);
    wait(.2);

    //GPO_IEN
    //sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x00, 0x01, 0x00, 0x04);   // GPO_IEN, RDSIEN
    //sendCommand(command, 6);
    //wait(.2);

    //FM_RDS_INT_SOURCE
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x15, 0x00, 0x00, 0x01);   //  RDSRECV
    sendCommand(command, 6);
    wait(.2);

    //FM_RDS_INT_FIFO_COUNT
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x15, 0x01, 0x00, 0x01);   // FIFIO count 1
    sendCommand(command, 6);
    wait(.2);

    //Enable RDS
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x15, 0x02, 0x00, 0x01);   // FM_RDS_CONFIG , RDSEN
    sendCommand(command, 6);
    wait(.2);

    //Set the seek band for the desired mode (AM and FM can use default values)
    switch(_mode){
        case SW:
            //Set the lower band limit for Short Wave Radio to 2300 kHz
            sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x34, 0x00, 0x08, 0xFC);
            sendCommand(command, 6);
            wait(.001);
            //Set the upper band limit for Short Wave Radio to 23000kHz
            sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x34, 0x01, 0x59, 0xD8);
            sendCommand(command, 6);            
            wait(.001);
            break;
        case LW:
            //Set the lower band limit for Long Wave Radio to 152 kHz
            sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x34, 0x00, 0x00, 0x99);
            sendCommand(command, 6);
            wait(.001);
            //Set the upper band limit for Long Wave Radio to 279 kHz
            sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x34, 0x01, 0x01, 0x17);
            sendCommand(command, 6);
            wait(.001);            
            break;
        default:
            break;
    }
}

/*
* Description: Tunes the Si4735 to a frequency
*
* Params: int frequency - The desired frequency in kHz
*
* Returns: True if tune was successful
*            False if tune was unsuccessful
*/
bool Si4735::tuneFrequency(int frequency){
    //Split the desired frequency into two character for use in the
    //set frequency command.
    char highByte = frequency >> 8;
    char lowByte = frequency & 0x00FF;
      
    //Depending on the current mode, set the new frequency.
    switch(_mode){
        case FM:
            sprintf(command, "%c%c%c%c", FM_TUNE_FREQ, 0x00, highByte, lowByte);
            break;
        case AM:
        case SW:
        case LW:
            sprintf(command, "%c%c%c%c", AM_TUNE_FREQ, 0x00, highByte, lowByte);
            break;
        default:
            break;
    }
    sendCommand(command, 4);
    wait(.1);
/** 
    sprintf(command, "%c%c%c", FM_AGC_OVERRIDE, 0x01, 0x00);
    sendCommand(command, 3);
    wait(.1);
**/   
    return true;
}

//This function gets the tuned-in frequency from chip and will be returned.
// A lower RSSI value compared to the value after start-up is due to AGC.
int Si4735::getFrequency(void){
    char data[8];
    int freq = 0;
    
    sprintf(command, "%c%c", FM_TUNE_STATUS, 0x00);
    sendCommand(command, 2);
    wait(.1);
    getResponse(data, 8);
    
    Status = data[0];
    Resp1  = data[1];
    freq = data[2];
    freq = (freq << 8) | data[3];
    //Frequency = (float)freq/100.0;
    Frequency = freq;
    RSSI = data[4];
    SNR  = data[5];
    getAGCstatus();
    return freq;
}

bool Si4735::seekUp(void){
    //Use the current mode selection to seek up.
    switch(_mode){
        case FM:
            sprintf(command, "%c%c", FM_SEEK_START, 0x0C);
            sendCommand(command, 2);
            break;
        case AM:
        case SW:
        case LW:
            sprintf(command, "%c%c%c%c%c%c", 0x41, 0x0C, 0x00, 0x00, 0x00, 0x00);
            sendCommand(command, 6);
            break;
        default:
            break;
    }
    wait(.001);
    return true;
}

bool Si4735::seekDown(void){
    //Use the current mode selection to seek down.
    switch(_mode){
        case FM:
            sprintf(command, "%c%c", FM_SEEK_START, 0x04);
            sendCommand(command, 2);
            break;
        case AM:
        case SW:
        case LW:
            sprintf(command, "%c%c%c%c%c%c", 0x41, 0x04, 0x00, 0x00, 0x00, 0x00);
            sendCommand(command, 6);
            break;
        default:
            break;
    }
    wait(.001);
    return true;
}

void Si4735::volumeUp(void){
    //If we're not at the maximum volume yet, increase the volume
    if(_currentVolume < 63){
        _currentVolume+=1;
        //Set the volume to the current value.
        sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x40, 0x00, 0x00, _currentVolume);
        sendCommand(command, 6);
        wait(.01);        
    }
}

void Si4735::volumeDown(void){
    //If we're not at the maximum volume yet, increase the volume
    if(_currentVolume > 0){
        _currentVolume-=1;
        //Set the volume to the current value.
        sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x40, 0x00, 0x00, _currentVolume);
        sendCommand(command, 6);
        wait(.01);        
    }
}

void Si4735::mute(void){
    //Disable Mute
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x40, 0x01, 0x00, 0x03);
    sendCommand(command, 6);
    wait(.001);
}

void Si4735::unmute(void){
    //Disable Mute
    sprintf(command, "%c%c%c%c%c%c", SET_PROPERTY, 0x00, 0x40, 0x01, 0x00, 0x00);
    sendCommand(command, 6);
    wait(.001);
}

bool Si4735::getValidSignal(void){
    char response[8];
    bool valid = false;
    
    sprintf(command, "%c%c", FM_RSQ_STATUS, 0x01);
    sendCommand(command, 2);
    wait(.001);
    getResponse (response,8);
    wait(.001);
    if (response[2] & 0x01) valid = true;        
    return valid;
}

char Si4735::getIntStatus(void){
    char response[2];
    
    sprintf(command, "%c", GET_INT_STATUS);
    sendCommand(command, 1);
    wait(.001);
    getResponse (response,1);
    wait(.001);        
    return response[0];
}

void Si4735::getProperty(int property, char * response){
    char arg[2];
    arg[0] = property >> 8;
    arg[1] = property & 0xFF;
    sprintf(command, "%c%c%c%c", GET_PROPERTY, 0x00, arg[0], arg[1]);
    sendCommand(command, 4);
    wait(.001);
    getResponse (response,4);
    wait(.001);        
    return;
}

void Si4735::getFmRDSStatus(char * response){

    sprintf(command, "%c%c", 0x24, 0x01);
    sendCommand(command, 2);
    wait(.001);
    getResponse (response,13);
    wait(.001);        
    return;
}

void Si4735::end(void){
    sprintf(command, "%c", 0x11);
    sendCommand(command, 1);
    wait(.001);
}

/*******************************************
*
* Private Functions
*
*******************************************/

void Si4735::sendCommand(char * command, int length){
  //ack = i2c_->write(address_write, command, length, false);
  i2c_->start();
  bool ack = i2c_->write(address_write);
  for(int i=0; i<length; i++)
    ack = i2c_->write(command[i]);        
  char CTS = i2c_->read(1);
  i2c_->stop();
 }
  
  void Si4735::getResponse(char * response, int length){
    int i = 0;
    i2c_->start();  
    i2c_->write(address_read);
    for (i=0; i<length-1; i++) {
        response[i] = i2c_->read(1);    //ACK
        //printf("%02X ",response[i]);
    }    
    response[i] = i2c_->read(0);        //NACK
    i2c_->stop();
    //printf("\n");
}

 void Si4735::getProperty(int property){
   char data[4];
   char highByte = property >> 8;
   char lowByte = property & 0x00FF;
   
    sprintf(command, "%c%c%c%c", GET_PROPERTY, 0x00, highByte, lowByte);
    sendCommand(command, 4);
    wait(.2);
    getResponse(data, 4);
    printf("Property:%02X,%02X\n",data[2],data[3]);
}    
 void Si4735::getAGCstatus(){
   char data[3];
   
    sprintf(command, "%c", FM_AGC_STATUS);
    sendCommand(command, 1);
    wait(.2);
    getResponse(data, 3);
    printf("AGC:%02X,%02X,%02X\n",data[0],data[1],data[2]);
}


